﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;

namespace NSEEOD_downloader.Tasks
{
    public class NSEEODTask : DownloadTask
    {
        public const string SETTINGS_PARSELINK = "NSE_EOD";
        private string linkTextForParsing = "";

        public NSEEODTask()
        {
            linkTextForParsing = Utils.GetString(Application.UserAppDataRegistry.GetValue(SETTINGS_PARSELINK));
        }

        private static string ConvertNSEEODLineToMetaStockASCIILine(string NSEEODLine)
        {
            string MSEODLine = "";
            try
            {
                //NSE:  0    ,  1   , 2  ,  3 , 4,  5   ,  6 ,   7     ,   8     ,   9     , 10,
                //NSE: SYMBOL,SERIES,OPEN,HIGH,LOW,CLOSE,LAST,PREVCLOSE,TOTTRDQTY,TOTTRDVAL,TIMESTAMP,
                //MS:     0    ,  1   , 2    ,  3   , 4   ,  5    ,  6  ,
                //MS:  <ticker>,<Date>,<open>,<high>,<low>,<close>,<vol>,
                string[] Values = NSEEODLine.Split(',');
                if (Values[1] != "EQ")
                {
                    return "";
                }

                StringBuilder MSLine = new StringBuilder();
                MSLine.Append(Values[0]); MSLine.Append(",");
                DateTime date = DateTime.Parse(Values[10]);
                MSLine.Append(date.ToString("yyyyMMdd")); MSLine.Append(",");
                MSLine.Append(Values[2]); MSLine.Append(",");
                MSLine.Append(Values[3]); MSLine.Append(",");
                MSLine.Append(Values[4]); MSLine.Append(",");
                MSLine.Append(Values[5]); MSLine.Append(",");
                MSLine.Append(Values[8]); MSLine.Append(",");
                MSEODLine = MSLine.ToString();
            }
            catch (Exception err)
            {
                Debug.WriteLine(err.Message);
                return "";
            }
            return MSEODLine;
        }

        protected override string ConvertDownloadedDataToCurrentTaskData(string NSEData)
        {
            StringBuilder Result = new StringBuilder();
            string[] Lines = NSEData.Split('\n');
            Result.AppendLine(Constants.META_STOCK_ASCII_HEADER);

            for (int line = 1; line < Lines.Length; line++)
            {
                if (Lines[line].Length < 4)
                {
                    continue;
                }
                string Converted = ConvertNSEEODLineToMetaStockASCIILine(Lines[line]);
                if (Converted.Length < 4)
                {
                    continue;
                }
                Result.AppendLine(Converted);
            }
            return Result.ToString();
        }

        protected override string ParseLink(string link,DateTime date)
        {
            return Utils.GetLinkFromFormatString(link, date);
        }

        protected override string GetLinkText()
        {
            return linkTextForParsing;
        }
    }
}
